@@ -41,10 +41,13 @@ static void aisleriot_slot_renderer_get_property (GObject *object,
 static void aisleriot_slot_renderer_paint (ClutterActor *actor);
 
 static void aisleriot_slot_renderer_set_cache (AisleriotSlotRenderer *srend,
-                                               AisleriotCardCache *cache);
+                                               GamesCardTexturesCache *cache);
 
 static void completed_cb (AisleriotSlotRenderer *srend);
 
+static const ClutterColor white = { 0xff, 0xff, 0xff, 0xff };
+static const ClutterColor default_highlight_color = { 0, 0, 0xaa, 0xff };
+
 G_DEFINE_TYPE (AisleriotSlotRenderer, aisleriot_slot_renderer,
                CLUTTER_TYPE_ACTOR);
 
@@ -56,10 +59,11 @@ typedef struct _AnimationData AnimationData;
 
 struct _AisleriotSlotRendererPrivate
 {
-  AisleriotCardCache *cache;
+  GamesCardTexturesCache *cache;
 
   Slot *slot;
 
+  ClutterColor highlight_color;
   gboolean show_highlight;
   gint highlight_start;
 
@@ -68,6 +72,7 @@ struct _AisleriotSlotRendererPrivate
   ClutterTimeline *timeline;
   guint completed_handler;
   GArray *animations;
+  guint n_unexposed_animated_cards;
 
   ClutterContainer *animation_layer;
 };
@@ -85,6 +90,7 @@ enum
   PROP_CACHE,
   PROP_SLOT,
   PROP_HIGHLIGHT,
+  PROP_HIGHLIGHT_COLOR,
   PROP_REVEALED_CARD,
   PROP_ANIMATION_LAYER
 };
@@ -105,7 +111,7 @@ aisleriot_slot_renderer_class_init (AisleriotSlotRendererClass *klass)
   actor_class->paint = aisleriot_slot_renderer_paint;
 
   pspec = g_param_spec_object ("cache", NULL, NULL,
-                               AISLERIOT_TYPE_CARD_CACHE,
+                               GAMES_TYPE_CARD_TEXTURES_CACHE,
                                G_PARAM_WRITABLE |
                                G_PARAM_CONSTRUCT_ONLY |
                                G_PARAM_STATIC_NAME |
@@ -122,7 +128,7 @@ aisleriot_slot_renderer_class_init (AisleriotSlotRendererClass *klass)
   g_object_class_install_property (gobject_class, PROP_SLOT, pspec);
 
   pspec = g_param_spec_int ("highlight", NULL, NULL,
-                            -1, G_MAXINT, -1,
+                            -1, G_MAXINT, G_MAXINT,
                             G_PARAM_WRITABLE |
                             G_PARAM_READABLE |
                             G_PARAM_STATIC_NAME |
@@ -130,8 +136,27 @@ aisleriot_slot_renderer_class_init (AisleriotSlotRendererClass *klass)
                             G_PARAM_STATIC_BLURB);
   g_object_class_install_property (gobject_class, PROP_HIGHLIGHT, pspec);
 
+
+#if 0
+  pspec = clutter_param_spec_color ("highlight-color", NULL, NULL,
+                                    &default_highlight_color,
+                                    G_PARAM_WRITABLE |
+                                    G_PARAM_STATIC_NAME |
+                                    G_PARAM_STATIC_NICK |
+                                    G_PARAM_STATIC_BLURB |
+                                    G_PARAM_CONSTRUCT);
+#else
+  pspec = g_param_spec_boxed ("highlight-color", NULL, NULL,
+                              CLUTTER_TYPE_COLOR,
+                              G_PARAM_WRITABLE |
+                              G_PARAM_STATIC_NAME |
+                              G_PARAM_STATIC_NICK |
+                              G_PARAM_STATIC_BLURB);
+#endif
+  g_object_class_install_property (gobject_class, PROP_HIGHLIGHT_COLOR, pspec);
+
   pspec = g_param_spec_int ("revealed-card", NULL, NULL,
-                            -1, G_MAXINT, 0,
+                            -1, G_MAXINT, -1,
                             G_PARAM_WRITABLE |
                             G_PARAM_READABLE |
                             G_PARAM_STATIC_NAME |
@@ -158,13 +183,14 @@ aisleriot_slot_renderer_init (AisleriotSlotRenderer *self)
 
   priv = self->priv = AISLERIOT_SLOT_RENDERER_GET_PRIVATE (self);
 
+  priv->revealed_card = -1;
   priv->highlight_start = G_MAXINT;
+  priv->highlight_color = default_highlight_color;
+
   priv->animations = g_array_new (FALSE, FALSE, sizeof (AnimationData));
   priv->timeline = clutter_timeline_new_for_duration (500);
   g_signal_connect_swapped (priv->timeline, "completed",
                             G_CALLBACK (completed_cb), self);
-
-  priv->revealed_card = -1;
 }
 
 static void
@@ -176,7 +202,7 @@ aisleriot_slot_renderer_dispose (GObject *object)
   aisleriot_slot_renderer_set_cache (self, NULL);
 
   /* Get rid of any running animations */
-  aisleriot_slot_renderer_set_animations (self, 0, NULL);
+  aisleriot_slot_renderer_set_animations (self, 0, NULL, 0);
 
   if (priv->timeline) {
     g_object_unref (priv->timeline);
@@ -200,7 +226,7 @@ aisleriot_slot_renderer_finalize (GObject *object)
 }
 
 ClutterActor *
-aisleriot_slot_renderer_new (AisleriotCardCache *cache, Slot *slot)
+aisleriot_slot_renderer_new (GamesCardTexturesCache *cache, Slot *slot)
 {
   ClutterActor *self = g_object_new (AISLERIOT_TYPE_SLOT_RENDERER,
                                      "cache", cache,
@@ -212,10 +238,13 @@ aisleriot_slot_renderer_new (AisleriotCardCache *cache, Slot *slot)
 
 static void
 aisleriot_slot_renderer_set_cache (AisleriotSlotRenderer *srend,
-                                   AisleriotCardCache *cache)
+                                   GamesCardTexturesCache *cache)
 {
   AisleriotSlotRendererPrivate *priv = srend->priv;
 
+  if (cache == priv->cache)
+    return;
+
   if (cache)
     g_object_ref (cache);
 
@@ -248,6 +277,11 @@ aisleriot_slot_renderer_set_property (GObject *object,
                                              g_value_get_int (value));
       break;
 
+    case PROP_HIGHLIGHT_COLOR:
+      aisleriot_slot_renderer_set_highlight_color (srend,
+                                                   g_value_get_boxed (value));
+      break;
+
     case PROP_REVEALED_CARD:
       aisleriot_slot_renderer_set_revealed_card (srend,
                                                  g_value_get_int (value));
@@ -288,6 +322,7 @@ aisleriot_slot_renderer_get_property (GObject *object,
                           aisleriot_slot_renderer_get_animation_layer (srend));
       break;
 
+    case PROP_HIGHLIGHT_COLOR:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -300,17 +335,13 @@ aisleriot_slot_renderer_paint_card (AisleriotSlotRenderer *srend,
 {
   AisleriotSlotRendererPrivate *priv = srend->priv;
   Card card = CARD (priv->slot->cards->data[card_num]);
-  gboolean is_highlighted;
   CoglHandle cogl_tex;
   guint tex_width, tex_height;
   int cardx, cardy;
-  static const ClutterColor white = { 0xff, 0xff, 0xff, 0xff };
 
-  is_highlighted = priv->show_highlight && (card_num >= priv->highlight_start);
-
-  cogl_tex = aisleriot_card_cache_get_card_texture (priv->cache,
-                                                    card,
-                                                    is_highlighted);
+  cogl_tex = games_card_textures_cache_get_card_texture (priv->cache, card);
+  if (G_UNLIKELY (cogl_tex == COGL_INVALID_HANDLE))
+    return;
 
   tex_width = cogl_texture_get_width (cogl_tex);
   tex_height = cogl_texture_get_height (cogl_tex);
@@ -326,6 +357,19 @@ aisleriot_slot_renderer_paint_card (AisleriotSlotRenderer *srend,
                           CLUTTER_INT_TO_FIXED (cardx + tex_width),
                           CLUTTER_INT_TO_FIXED (cardy + tex_height),
                           0, 0, CFX_ONE, CFX_ONE);
+
+  if (priv->show_highlight && (card_num >= priv->highlight_start)) {
+    ClutterColor color = priv->highlight_color;
+
+    color.alpha = 0x80;
+    cogl_color (&color);
+    cogl_texture_rectangle (cogl_tex,
+                            CLUTTER_INT_TO_FIXED (cardx),
+                            CLUTTER_INT_TO_FIXED (cardy),
+                            CLUTTER_INT_TO_FIXED (cardx + tex_width),
+                            CLUTTER_INT_TO_FIXED (cardy + tex_height),
+                            0, 0, CFX_ONE, CFX_ONE);
+  }
 }
 
 static void
@@ -333,7 +377,7 @@ aisleriot_slot_renderer_paint (ClutterActor *actor)
 {
   AisleriotSlotRenderer *srend = (AisleriotSlotRenderer *) actor;
   AisleriotSlotRendererPrivate *priv = srend->priv;
-  guint n_cards;
+  guint n_cards, n_animated_cards;
   guint8 *cards;
   guint i;
 
@@ -342,17 +386,20 @@ aisleriot_slot_renderer_paint (ClutterActor *actor)
 
   cards = priv->slot->cards->data;
   n_cards = priv->slot->cards->len;
+  n_animated_cards = priv->animations->len + priv->n_unexposed_animated_cards;
 
   g_assert (n_cards >= priv->slot->exposed);
-  g_assert (n_cards >= priv->animations->len);
+  g_assert (priv->n_unexposed_animated_cards == 0 || priv->animations->len > 0);
+  g_assert (n_cards >= n_animated_cards);
 
-  if (priv->slot->cards->len <= priv->animations->len) {
+  if (n_cards <= n_animated_cards) {
     CoglHandle cogl_tex;
     guint tex_width, tex_height;
-    static const ClutterColor white = { 0xff, 0xff, 0xff, 0xff };
 
-    cogl_tex = aisleriot_card_cache_get_slot_texture (priv->cache,
-                                                      priv->show_highlight);
+    cogl_tex = games_card_textures_cache_get_slot_texture (priv->cache);
+    if (G_UNLIKELY (cogl_tex == COGL_INVALID_HANDLE))
+      return;
+
     tex_width = cogl_texture_get_width (cogl_tex);
     tex_height = cogl_texture_get_height (cogl_tex);
 
@@ -362,21 +409,29 @@ aisleriot_slot_renderer_paint (ClutterActor *actor)
                             CLUTTER_INT_TO_FIXED (tex_width),
                             CLUTTER_INT_TO_FIXED (tex_height),
                             0, 0, CFX_ONE, CFX_ONE);
-  }
 
-  if (n_cards > priv->animations->len) {
+    if (priv->show_highlight) {
+      cogl_color (&priv->highlight_color);
+      cogl_texture_rectangle (cogl_tex,
+                              0, 0,
+                              CLUTTER_INT_TO_FIXED (tex_width),
+                              CLUTTER_INT_TO_FIXED (tex_height),
+                              0, 0, CFX_ONE, CFX_ONE);
+    }
+  } else {
     guint first_card, last_card;
 
-    first_card = MIN (n_cards - priv->animations->len - 1,
+    first_card = MIN (n_cards - n_animated_cards - 1,
                       n_cards - priv->slot->exposed);
-    last_card = n_cards - priv->animations->len;
+    last_card = n_cards - n_animated_cards;
 
     for (i = first_card; i < last_card; i++)
       if (i != priv->revealed_card)
         aisleriot_slot_renderer_paint_card (srend, i);
 
     /* Paint the revealed card after all of the other cards so that it
-       will appeear on top */
+     * will appeear on top.
+     */
     if (priv->revealed_card >= first_card && priv->revealed_card < last_card)
       aisleriot_slot_renderer_paint_card (srend, priv->revealed_card);
   }
@@ -408,6 +463,22 @@ aisleriot_slot_renderer_set_highlight (AisleriotSlotRenderer *srend,
   g_object_notify (G_OBJECT (srend), "highlight");
 }
 
+void
+aisleriot_slot_renderer_set_highlight_color (AisleriotSlotRenderer *srend,
+                                             const ClutterColor *color)
+{
+  AisleriotSlotRendererPrivate *priv = srend->priv;
+
+  g_return_if_fail (color != NULL);
+
+  if (clutter_color_equal (color, &priv->highlight_color))
+    return;
+
+  priv->highlight_color = *color;
+
+  g_object_notify (G_OBJECT (srend), "highlight-color");
+}
+
 gint
 aisleriot_slot_renderer_get_revealed_card (AisleriotSlotRenderer *srend)
 {
@@ -469,7 +540,8 @@ aisleriot_slot_renderer_set_animation_layer (AisleriotSlotRenderer *srend,
 void
 aisleriot_slot_renderer_set_animations (AisleriotSlotRenderer *srend,
                                         guint n_anims,
-                                        const AisleriotAnimStart *anims)
+                                        const AisleriotAnimStart *anims,
+                                        guint n_unexposed_animated_cards)
 {
   AisleriotSlotRendererPrivate *priv;
   guint i;
@@ -508,20 +580,20 @@ aisleriot_slot_renderer_set_animations (AisleriotSlotRenderer *srend,
     ClutterKnot knots[2];
     Card card = CARD (priv->slot->cards->data[card_num]);
     guint card_width, card_height;
-    CoglHandle cogl_tex;
 
     memset (&anim_data, 0, sizeof (anim_data));
 
-    anim_data.card_tex = aisleriot_card_new (priv->cache, card);
+    anim_data.card_tex = aisleriot_card_new (priv->cache, anims[i].old_card,
+                                             card, &priv->highlight_color);
+
+    card_width = clutter_actor_get_width (anim_data.card_tex);
+    card_height = clutter_actor_get_height (anim_data.card_tex);
+
     g_object_ref_sink (anim_data.card_tex);
     if (priv->animation_layer)
       clutter_container_add (priv->animation_layer,
                              CLUTTER_ACTOR (anim_data.card_tex), NULL);
 
-    cogl_tex = aisleriot_card_cache_get_card_texture (priv->cache, card, FALSE);
-    card_width = cogl_texture_get_width (cogl_tex);
-    card_height = cogl_texture_get_height (cogl_tex);
-
     clutter_actor_set_position (anim_data.card_tex,
                                 anims[i].cardx, anims[i].cardy);
 
@@ -540,9 +612,11 @@ aisleriot_slot_renderer_set_animations (AisleriotSlotRenderer *srend,
                                                  G_N_ELEMENTS (knots));
     clutter_behaviour_apply (anim_data.move, anim_data.card_tex);
 
-    if (anims[i].face_down != card.attr.face_down) {
-      gint center_x = card_width / 2;
-      gint center_y = card_height / 2;
+    if (anims[i].old_card.value != card.value) {
+      int center_x, center_y;
+
+      center_x = card_width / 2;
+      center_y = card_height / 2;
 
       clutter_actor_set_rotation (anim_data.card_tex, CLUTTER_Y_AXIS,
                                   180.0,
@@ -578,6 +652,8 @@ aisleriot_slot_renderer_set_animations (AisleriotSlotRenderer *srend,
     clutter_timeline_start (priv->timeline);
   }
 
+  priv->n_unexposed_animated_cards = n_unexposed_animated_cards;
+
   clutter_actor_queue_redraw (CLUTTER_ACTOR (srend));
 }
 
@@ -585,7 +661,7 @@ static void
 completed_cb (AisleriotSlotRenderer *srend)
 {
   /* Get rid of all animation actors */
-  aisleriot_slot_renderer_set_animations (srend, 0, NULL);
+  aisleriot_slot_renderer_set_animations (srend, 0, NULL, 0);
 
   /* Redraw so that the animated actors will be drawn as part of the
      renderer instead */
