@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 __author__ = 'Robert Ancell <bob27@users.sourceforge.net>'
 __license__ = 'GNU General Public License Version 2'
 __copyright__ = 'Copyright 2005-2006  Robert Ancell'
@@ -58,6 +59,9 @@ class CECPProtocol:
         """Called when the AI resigns"""
         print 'CECP AI resigns'
         
+    def logText(self, text, style):
+        print 'LOG: %s' % text
+        
     # Public methods
     
     def sendSetSearchDepth(self, searchDepth):
@@ -92,6 +96,10 @@ class CECPProtocol:
         """Stop the AI from automatically moving"""
         self.onOutgoingData('force\n')
         
+    def sendUndo(self):
+        """Undo the last move"""
+        self.onOutgoingData('undo\n')
+        
     def sendMovePrompt(self):
         """Get the AI to move for the current player"""
         self.onOutgoingData('go\n')
@@ -202,6 +210,11 @@ class Connection(CECPProtocol):
         # Prompt the AI to move
         self.sendMovePrompt()
         
+    def undoMove(self):
+        """Undo the last move made by this AI"""
+        self.sendWait()
+        self.sendUndo()
+
     def reportMove(self, move, isSelf):
         """Report the move the current player has made.
         
@@ -227,8 +240,3 @@ class Connection(CECPProtocol):
     def onIllegalMove(self, move):
         """Called by CECPProtocol"""
         print 'CECP illegal move: ' + move
-
-if __name__ == '__main__':
-    c = CECPConnection('gnuchess')
-    while True:
-        c.read()
