@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 __author__ = 'Robert Ancell <bob27@users.sourceforge.net>'
 __license__ = 'GNU General Public License Version 2'
 __copyright__ = 'Copyright 2005-2006  Robert Ancell'
@@ -235,28 +236,33 @@ class GtkNewGameDialog:
             self.__setCombo('black_difficulty_combo', blackLevel)
 
         # Use supplied settings
-        errorText = ''
+        errors = []
         g = self.game
         if g is not None:
             self.__gui.get_widget('game_name_entry').set_text(g.name)
             self.__customName = True
+            # Translators: Error displayed when unable to load a game due to
+            # the require game engine not being available. %s is replaced with
+            # the name of the missing engine.
+            noEngineFormat = _('Unable to find %s engine')
             if not self.__setCombo('white_type_combo', g.white.type):
-                errorText += _('Unable to find %s engine\n') % repr(g.white.type)
+                errors.append(noEngineFormat % repr(g.white.type))
             self.__setCombo('white_difficulty_combo', g.white.level)
             if not self.__setCombo('black_type_combo', g.black.type):
-                errorText += _('Unable to find %s engine\n') % repr(g.black.type)
+                errors.append(noEngineFormat % repr(g.black.type))
             self.__setCombo('black_difficulty_combo', g.black.level)
             # TODO: Others
             
             # Change title for loaded games
             if g.path is not None:
-                # Translators: This is the title of the dialog when continuing a loaded game
+                # Translators: New Game Dialog: Title of the dialog when continuing a loaded game
                 self.window.set_title(_('Configure loaded game (%i moves)') % len(g.moves))
 
         # Display warning if missing the AIs
-        if len(errorText) > 0:
+        if len(errors) > 0:
+            # Translators: New Game Dialog: Title of error box when loaded game had AI engines missing
             self.__gui.get_widget('info_title_label').set_markup('<big><b>%s</b></big>' % _('Game settings changed'))
-            self.__gui.get_widget('info_description_label').set_markup('<i>%s</i>' % errorText[:-1])
+            self.__gui.get_widget('info_description_label').set_markup('<i>%s</i>' % '\n'.join(errors))
             self.__gui.get_widget('info_box').show()
 
         # Show the dialog
@@ -432,12 +438,14 @@ class GtkLoadGameDialog:
 
         # Filter out non PGN files by default
         pgnFilter = gtk.FileFilter()
-        pgnFilter.set_name('PGN files')
+        # Translators: Load Game Dialog: Name of filter to show only PGN files
+        pgnFilter.set_name(_('PGN files'))
         pgnFilter.add_pattern('*.pgn')
         fileChooser.add_filter(pgnFilter)
         
         allFilter = gtk.FileFilter()
-        allFilter.set_name('All files')
+        # Translators: Load Game Dialog: Name of filter to show all files
+        allFilter.set_name(_('All files'))
         allFilter.add_pattern('*')
         fileChooser.add_filter(allFilter)
         
@@ -458,7 +466,7 @@ class GtkLoadGameDialog:
 
             fileName = self.__gui.get_widget('filechooserwidget').get_filename()
             if fileName is None:
-                # Translators: Message displayed in load game dialog when no file is selected
+                # Translators: Load Game Dialog: Message displayed when no file is selected
                 error = _('Please select a file to load')
             else:
                 error = self.__mainUI.feedback.loadGame(fileName, responseId == gtk.RESPONSE_YES)
@@ -577,10 +585,12 @@ class GtkPreferencesDialog:
         widget.set_model(moveModel)
                         # Translators: Move Format Combo: Moves shown in human descriptive notation
         move_formats = [('human', _('Human')),
-                        # Translators: Move Format Combo: Moves shown in long algebraic notation (LAN)
-                        ('lan', _('Long Algebraic')),
                         # Translators: Move Format Combo: Moves shown in standard algebraic notation (SAN)
-                        ('san', _('Standard Algebraic'))]
+                        ('san', _('Standard Algebraic')),
+                        # Translators: Move Format Combo: Moves shown in standard figurine algebraic notation (FAN)
+                        ('fan', _('Figurine')),
+                        # Translators: Move Format Combo: Moves shown in long algebraic notation (LAN)
+                        ('lan', _('Long Algebraic'))]
         for (key, label) in move_formats:
             iter = moveModel.append()
             moveModel.set(iter, 0, label, 1, key)
@@ -605,20 +615,24 @@ class GtkPreferencesDialog:
         promotionModel = gtk.ListStore(str, str)
         widget = self.__gui.get_widget('promotion_type_combo')
         widget.set_model(promotionModel)
-                          # Translators: Promotion Combo: Promote to a queen
-        promotion_list = [('queen', _('Queen')),
-                          # Translators: Promotion Combo: Promote to a knight
-                          ('knight', _('Knight')),
-                          # Translators: Promotion Combo: Promote to a rook
-                          ('rook', _('Rook')),
-                          # Translators: Promotion Combo: Promote to a bishop
-                          ('bishop', _('Bishop'))]
+                          # Translators: Promotion Combo: Promote to a queen. Do not translate the 'chess-piece|' text.
+        promotion_list = [('queen',  _('chess-piece|Queen')),
+                          # Translators: Promotion Combo: Promote to a knight. Do not translate the 'chess-piece|' text.
+                          ('knight', _('chess-piece|Knight')),
+                          # Translators: Promotion Combo: Promote to a rook. Do not translate the 'chess-piece|' text.
+                          ('rook',   _('chess-piece|Rook')),
+                          # Translators: Promotion Combo: Promote to a bishop. Do not translate the 'chess-piece|' text.
+                          ('bishop', _('chess-piece|Bishop'))]
         for (key, label) in promotion_list:
+            try:
+                label = label.split('|', 1)[1]
+            except IndexError:
+                pass
             iter = promotionModel.append()
             promotionModel.set(iter, 0, label, 1, key)
             
         # Watch for config changes
-        for key in ['show_3d', 'show_toolbar', 'show_history', 
+        for key in ['show_3d', 'show_3d_smooth', 'show_toolbar', 'show_history', 
                     'show_move_hints', 'show_numbering',
                     'move_format', 'board_view', 'promotion_type']:
             glchess.config.watch(key, self.__applyConfig)
@@ -634,7 +648,11 @@ class GtkPreferencesDialog:
         """        
         if name == 'show_3d':
             self.__gui.get_widget('show_3d').set_active(value)
+            self.__gui.get_widget('show_3d_smooth').set_sensitive(value)            
 
+        elif name == 'show_3d_smooth':
+            self.__gui.get_widget('show_3d_smooth').set_active(value)
+            
         elif name == 'show_toolbar':
             self.__gui.get_widget('show_toolbar').set_active(value)
                 
@@ -717,6 +735,10 @@ class GtkPreferencesDialog:
     def _on_3d_view_activate(self, widget):
         """Gtk+ callback"""
         glchess.config.set('show_3d', widget.get_active())
+        
+    def _on_3d_smooth_toggled(self, widget):
+        """Gtk+ callback"""
+        glchess.config.set('show_3d_smooth', widget.get_active())        
 
     def _on_show_toolbar_activate(self, widget):
         """Gtk+ callback"""
