@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import gettext
 
 import gobject
@@ -9,19 +10,95 @@ import glchess.ui
 
 _ = gettext.gettext
 
+class GtkNetworkAddDialog:
+    def __init__(self, networkDialog, parent):
+        self.__networkDialog = networkDialog
+
+        # Load the UI
+        self.__gui = gtkui.loadGladeFile('network_game.glade', 'add_account_dialog')
+        self.__gui.signal_autoconnect(self)
+        
+        self.__gui.get_widget('add_account_dialog').set_transient_for(parent)
+        
+        # FIXME: Hard-coded servers       
+        # name, host, port
+        self.serverModel = gtk.ListStore(str, str, int)
+        # Translators: Add Network Profile Dialog: Connect to the GGZ Gaming Zone server (the default)
+        self.serverModel.set(self.serverModel.append(), 0, _("GGZ Gaming Zone"), 1, "gnome.ggzgamingzone.org", 2, 5688)
+        # Translators: Add Network Profile Dialog: Use a custom server
+        self.serverModel.set(self.serverModel.append(), 0, _("Custom"), 1, "", 2, 5688)
+        
+        widget = self.__gui.get_widget('server_combo')
+        widget.set_model(self.serverModel)
+        cell = gtk.CellRendererText()
+        widget.pack_start(cell, False)
+        widget.add_attribute(cell, 'text', 0)
+        widget.set_model(self.serverModel)
+        widget.set_active(0)
+
+    def setVisible(self, isVisible):
+        widget = self.__gui.get_widget('add_account_dialog')
+        if isVisible:
+            widget.present()
+        else:
+            widget.hide()
+            self.clear()
+            
+    def clear(self):
+        self.__gui.get_widget('server_combo').set_active(0)
+        self.__gui.get_widget('username_entry').set_text('')        
+            
+    def _on_server_changed(self, widget):
+        widget = self.__gui.get_widget('server_combo')
+        model = widget.get_model()
+        iter = widget.get_active_iter()
+        (host,) = model.get(iter, 1)
+        (port,) = model.get(iter, 2)
+        self.__gui.get_widget('host_entry').set_text(host)
+        self.__gui.get_widget('port_spin').set_value(port)
+        table = self.__gui.get_widget('custom_server_table')
+        if host == '':
+            table.show()
+        else:
+            table.hide()
+            
+    def have_data(self):
+        username = self.__gui.get_widget('username_entry').get_text()
+        host = self.__gui.get_widget('host_entry').get_text()
+        return username != '' and host != ''
+
+    def _on_input_changed(self, widget):
+        self.__gui.get_widget('add_button').set_sensitive(self.have_data())
+
+    def _on_username_activate(self, widget):
+        if self.have_data():
+            self._on_response(None, gtk.RESPONSE_OK)
+
+    def _on_response(self, widget, response_id):
+        username = self.__gui.get_widget('username_entry').get_text()
+        host = self.__gui.get_widget('host_entry').get_text()
+        port = self.__gui.get_widget('port_spin').get_value_as_int()
+        name = '%s@%s' % (username, host) # FIXME
+        
+        if response_id == gtk.RESPONSE_OK:
+            profile = self.__networkDialog.feedback.addProfile((name, username, host, port))
+            self.__networkDialog.addProfile(profile, profile.name, useNow = True)
+        
+        self.__gui.get_widget('add_account_dialog').hide()
+        self.clear()
+            
+    def _on_delete(self, widget, event):
+        # Hide; don't delete this window
+        return True
+
 class GtkNetworkGameDialog(glchess.ui.NetworkController):
     """
     """
-    # The main UI and the ???
-    __mainUI = None
-    __gui = None
-
-    def __init__(self, mainUI, feedback, aiModel):
+    def __init__(self, mainUI, feedback):
         """Constructor for a new game dialog.
         
         'mainUI' is the main UI.
         'feedback' is the object to feedback events with.
-        'aiModel' is the AI models to use.
         """
         self.__mainUI = mainUI
         self.feedback = feedback
@@ -30,16 +107,26 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         self.__gui = gtkui.loadGladeFile('network_game.glade', 'network_game_dialog')
         self.__gui.signal_autoconnect(self)
         
-        self.profileModel = gtk.ListStore(gobject.TYPE_PYOBJECT, str)
-        iter = self.profileModel.append()
-        self.profileModel.set(iter, 0, None, 1, 'Disconnected')
+        # Selected profile
+        self.__profile = None
         
+        self.profileModel = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, str)
+        # Translators: Server Combo Box: Not connected to a server
+        self.profileModel.set(self.profileModel.append(), 0, None, 1, self._set_profile, 2, _('Disconnected'))
+        self.profileModelSuffixCount = 0
+        self.profileModel.set(self.profileModel.append(), 1, None)
+        self.profileModelSuffixCount += 1
+        # Translators: Server Combo Box: Add new profile
+        self.profileModel.set(self.profileModel.append(), 0, None, 1, self._new_profile, 2, _('New profile...'))
+        self.profileModelSuffixCount += 1
+
         widget = self.__gui.get_widget('server_combo')
         widget.set_model(self.profileModel)
+        widget.set_active(0)
+        widget.set_row_separator_func(self._is_profile_model_separator)
         cell = gtk.CellRendererText()
         widget.pack_start(cell, False)
-        widget.add_attribute(cell, 'text', 1)
-        #FIXME: delay this: widget.set_active_iter(iter)
+        widget.add_attribute(cell, 'text', 2)
 
         # room object, index, name, num players, description, font weight, font style, icon_name
         self.roomModel = gtk.TreeStore(gobject.TYPE_PYOBJECT, int, str, str, str, int, int, str)
@@ -48,7 +135,7 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         view = self.__gui.get_widget('room_list')
         view.set_model(self.roomModel)
         cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_('#'), cell)
+        column = gtk.TreeViewColumn('', cell)
         column.add_attribute(cell, 'text', 3)
         view.append_column(column)
         cell = gtk.CellRendererPixbuf()
@@ -56,13 +143,13 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         column.add_attribute(cell, 'icon-name', 7)
         view.append_column(column)
         cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_('Room'), cell)
+        column = gtk.TreeViewColumn('', cell)
         column.add_attribute(cell, 'text', 2)
         column.add_attribute(cell, 'weight', 5)
         column.add_attribute(cell, 'style', 6)
         view.append_column(column)
         cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_('Description'), cell)
+        column = gtk.TreeViewColumn('', cell)
         column.add_attribute(cell, 'text', 4)
         #view.append_column(column)
         view.connect('row-activated', self._on_room_changed)
@@ -76,7 +163,7 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         column.add_attribute(cell, 'icon-name', 2)
         view.append_column(column)
         cell = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_('Player'), cell)
+        column = gtk.TreeViewColumn('', cell)
         column.add_attribute(cell, 'text', 1)
         view.append_column(column)
 
@@ -89,23 +176,28 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         view.set_model(self.tableModel)
         
         cell = gtk.CellRendererText()
+        # Translators: Available GGZ Tables: Table name column title
         column = gtk.TreeViewColumn(_('Table'), cell)
         column.add_attribute(cell, 'text', 1)
         view.append_column(column)
         cell = gtk.CellRendererText()
+        # Translators: Available GGZ Tables: Seat status column title
         column = gtk.TreeViewColumn(_('Seats'), cell)
         column.add_attribute(cell, 'text', 2)
         view.append_column(column)
         cell = gtk.CellRendererText()
+        # Translators: Available GGZ Tables: Table description column title        
         column = gtk.TreeViewColumn(_('Description'), cell)
         column.add_attribute(cell, 'text', 3)
         view.append_column(column)
 
         view = self.__gui.get_widget('seat_list')
         cell = gtk.CellRendererText()
+        # Translators: Current GGZ Table: Seat name column title
         column = gtk.TreeViewColumn(_('Seat'), cell)
         column.add_attribute(cell, 'text', 2)
         view.append_column(column)
+        # Translators: Current GGZ Table: Player name column title        
         column = gtk.TreeViewColumn(_('Player'), cell)
         column.add_attribute(cell, 'text', 3)
         column.add_attribute(cell, 'style', 4)
@@ -124,6 +216,8 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         buffer.create_mark('end', buffer.get_end_iter())
         
         mainUI.setTooltipStyle(self.__gui.get_widget('info_panel'))
+        
+        self.__addProfileDialog = GtkNetworkAddDialog(self, self.__gui.get_widget('network_game_dialog'))
 
     # Extended methods
         
@@ -131,8 +225,14 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         """Called by glchess.ui.NetworkController"""
         widget = self.__gui.get_widget('network_game_dialog')
         if isVisible:
-            widget.show()
+            widget.present()
+            
+            # Prompt for new profile if none configured
+            # FIXME: Make this clearer this is the count of non-profile elements in the combo
+            if len(self.profileModel) <= (self.profileModelSuffixCount + 1):
+                self.__addProfileDialog.setVisible(True)
         else:
+            self.__addProfileDialog.setVisible(False)
             widget.hide()
             
     def setSensitive(self, isSensitive):
@@ -147,16 +247,22 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
     def clearError(self):
         self.__gui.get_widget('info_panel').hide()
 
-    def addProfile(self, profile, name):
+    def addProfile(self, profile, name, useNow = False):
         """Called by glchess.ui.UIController"""
-        iter = self.profileModel.append()
-        self.profileModel.set(iter, 0, profile, 1, name)
+        iter = self.profileModel.insert(len(self.profileModel) - self.profileModelSuffixCount)
+        self.profileModel.set(iter, 0, profile, 1, self._set_profile, 2, name)        
+        if self.__profile is None and useNow:
+            self.__gui.get_widget('server_combo').set_active_iter(iter)
 
     def setBusy(self, isBusy):
         """Called by glchess.ui.UIController"""
         if self._throbberTimer is not None:
             gobject.source_remove(self._throbberTimer)
             self._throbberTimer = None
+
+        # Disable room buttons when busy
+        widget = self.__gui.get_widget('room_button_box')
+        widget.set_sensitive(not isBusy)
         
         # Display animating frames if busy or idle frame if not
         if isBusy:
@@ -191,7 +297,7 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
 
         self.roomIters[room] = iter
         self.roomModel.set(iter, 0, room, 1, index, 2, name, 3, nPlayers, 4, description, 5, pango.WEIGHT_NORMAL, 6, style, 7, icon)
-        
+
     def enterRoom(self, room):
         """Called by glchess.ui.UIController"""
         for (r, iter) in self.roomIters.iteritems():
@@ -213,7 +319,7 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         """Called by glchess.ui.UIController"""
         iter = self.roomModel.get_iter_first()
         while iter is not None:
-            if table is self.roomModel.get_value(iter, 0):
+            if room is self.roomModel.get_value(iter, 0):
                 self.roomModel.remove(iter)
                 return
             iter = self.roomModel.iter_next(iter)
@@ -248,10 +354,13 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
             iter = seatModel.append()
             
         if number == 0:
+            # Translators: GGZ seat is occupied by the white player
             seatName = _('White')
         elif number == 1:
+            # Translators: GGZ seat is occupied by the black player            
             seatName = _('Black')
         else:
+            # Translators: GGZ seat is occupied by a spectator
             seatName = _('Spectator')
 
         style = pango.STYLE_ITALIC
@@ -259,11 +368,16 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         if type == 'player':
             style = pango.STYLE_NORMAL
         elif type == 'reserved':
-            occupant = _('Reserved for %s' % name)
+            # Translators: GGZ seat status: This seat is reserved. %s is replaced with
+            # the name of the player the seat is reserved for.
+            occupant = _('Reserved for %s') % name
         elif type == 'open':
+            # Translators: GGZ seat status: This seat is not taken
             occupant = _('Seat empty')
         elif type == 'bot':
-            occupant = _('AI (%s)' % name)
+            # Translators: GGZ seat status: This seat contains an AI player.
+            # %s is replaced with the name of the AI.
+            occupant = _('AI (%s)') % name
 
         seatModel.set(iter, 0, number, 1, type, 2, seatName, 3, occupant, 4, style)
 
@@ -325,6 +439,10 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         if atBottom:
             view.scroll_mark_onscreen(mark)
 
+    def clearText(self):
+        buffer = self.__gui.get_widget('chat_textview').get_buffer()
+        buffer.delete(buffer.get_start_iter(), buffer.get_end_iter())
+
     def close(self):
         """Called by glchess.ui.UIController"""
         self.__gui.get_widget('network_game_dialog').hide()        
@@ -335,26 +453,42 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         self._throbberTimer = None
         theme = gtk.icon_theme_get_default()
         size = 32
-        icon = theme.load_icon('process-working', size, 0)
         self._throbberFrames = []
-        for i in xrange(icon.get_height() / size):
-            for j in xrange(icon.get_width() / size):
-                frame = icon.subpixbuf(j * size, i * size, size, size)
-                self._throbberFrames.append(frame)
-                
-        # The throbber image should contain 1 idle image and N animation images.
-        # If the image is invalid (i.e. there is only one 32x32 icon) then duplicate
-        # that image as a 1 frame animation.
-        if len(self._throbberFrames) == 1:
-            self.throbberFrames.append(self._throbberFrames[0])
+        
+        try:
+            icon = theme.load_icon('process-idle', size, 0)
+        except gobject.GError:
+            pass
+        else:
+            self._throbberFrames.append(icon)
 
+        try:
+            icon = theme.load_icon('process-working', size, 0)
+        except gobject.GError:
+            pass
+        else:
+            # If a smaller icon was received than expected then use that size
+            height = icon.get_height()
+            width = icon.get_width()
+            size = min(size, height, width)
+
+            for i in xrange(height / size):
+                for j in xrange(width / size):
+                    frame = icon.subpixbuf(j * size, i * size, size, size)
+                    self._throbberFrames.append(frame)
+                
         # Display idle frame
         self._throbberFrame = 0
         self._updateThrobber()
 
     def _updateThrobber(self):
         widget = self.__gui.get_widget('throbber_image')
-        widget.set_from_pixbuf(self._throbberFrames[self._throbberFrame])
+        try:
+            icon = self._throbberFrames[self._throbberFrame]
+        except IndexError:
+            pass
+        else:
+            widget.set_from_pixbuf(icon)
         
         # Move to next frame restarting animation after idle frame
         self._throbberFrame += 1
@@ -397,7 +531,7 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         
         # Get the players
         game.white.type  = self.__getComboData(self.__gui.get_widget('white_type_combo'), 0)
-        if game.white.type is '':
+        if game.white.type == '':
             game.white.name = _('White')
         else:
             game.white.name = self.__getComboData(self.__gui.get_widget('white_type_combo'), 2)
@@ -411,7 +545,7 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
 
         game.duration = self.__getComboData(self.__gui.get_widget('time_combo'), 1)
         if game.duration < 0:
-            multplier = self.__getComboData(self.__gui.get_widget('custom_time_units_combo'), 1)
+            multiplier = self.__getComboData(self.__gui.get_widget('custom_time_units_combo'), 1)
             game.duration = self.__getComboData(self.__gui.get_widget('custom_time_spin'), 1) * multiplier
             
         # Save properties
@@ -423,6 +557,26 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         # Inform the child class
         self.__mainUI.feedback.onGameStart(game)
         
+    def _is_profile_model_separator(self, model, iter):
+        return model.get(iter, 1)[0] is None
+
+    def _set_profile(self, profile):
+        if profile != self.__profile:
+            self.__profile = profile
+            self.feedback.setProfile(profile)
+            
+    def __selectActiveProfile(self):
+        iter = self.profileModel.get_iter_first()
+        while iter is not None:
+            if self.__profile == self.profileModel.get_value(iter, 0):
+                break
+            iter = self.profileModel.iter_next(iter)
+        self.__gui.get_widget('server_combo').set_active_iter(iter)
+
+    def _new_profile(self, profile):
+        self.__selectActiveProfile()
+        self.__addProfileDialog.setVisible(True)
+
     # Gtk+ signal handlers
     
     def _on_table_selected(self, selection):
@@ -435,16 +589,17 @@ class GtkNetworkGameDialog(glchess.ui.NetworkController):
         widget = self.__gui.get_widget('table_join_button')
         widget.set_sensitive(isSensitive)
         
-    def _on_table_list_activated():
+    def _on_table_list_activated(self):
         pass
     
     def _on_server_combo_changed(self, widget):
         """Gtk+ callback"""
         model = widget.get_model()
         iter = widget.get_active_iter()
+        (method,) = model.get(iter, 1)
         (profile,) = model.get(iter, 0)
-        self.feedback.setProfile(profile)
-    
+        method(profile)
+
     def _on_chat_entry_activate(self, widget):
         """Gtk+ callback"""
         text = widget.get_text()
