@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #!/usr/bin/env python
 # png.py - PNG encoder in pure Python
 # Copyright (C) 2006 Johann C. Rocholl <johann@browsershots.org>
@@ -50,9 +51,9 @@ prompt for usage and a list of options.
 """
 
 
-__revision__ = '$Rev: 7989 $'
-__date__ = '$Date: 2008-10-08 23:17:53 -0500 (Wed, 08 Oct 2008) $'
-__author__ = '$Author: rancell $'
+__revision__ = '$Rev$'
+__date__ = '$Date$'
+__author__ = '$Author$'
 
 
 import sys
