@@ -23,18 +23,20 @@
  * USA
  */
 
+#include <config.h>
+#include <sys/types.h>
+#include <unistd.h>
 
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
-#include "config.h"
-#include <gnome.h>
 #include <pwd.h>
-
 #include <ggzmod.h>
 #include <ggz-embed.h>
 #include <ggz-gtk.h>
 
-#include <games-dlg-chat.h>
-#include <games-dlg-players.h>
+#include <libgames-support/games-dlg-chat.h>
+#include <libgames-support/games-dlg-players.h>
 
 #include "connectx_client.h"
 #include "connect4.h"
