@@ -23,9 +23,11 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <games-frame.h>
-#include <games-controls.h>
+
+#include <libgames-support/games-frame.h>
+#include <libgames-support/games-controls.h>
 
 #ifdef GGZ_CLIENT
 #include "ggz-network.h"
@@ -121,19 +123,27 @@ random_order_cb (GtkWidget * widget, gpointer data)
 static void
 fake_bonus_cb (GtkWidget * widget, gpointer data)
 {
+  gboolean value;
+
   if (!pref_dialog)
     return;
 
-  gnibbles_properties_set_fakes (GPOINTER_TO_INT (data));
+  value = GTK_TOGGLE_BUTTON (widget)->active;
+
+  gnibbles_properties_set_fakes (value);
 }
 
 static void
 sound_cb (GtkWidget * widget, gpointer data)
 {
+  gboolean value;
+
   if (!pref_dialog)
     return;
 
-  gnibbles_properties_set_sound (GPOINTER_TO_INT (data));
+  value = GTK_TOGGLE_BUTTON (widget)->active;
+
+  gnibbles_properties_set_sound (value);
 }
 
 static void
@@ -262,7 +272,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   if (properties->gamespeed == 4)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+  g_signal_connect (GTK_OBJECT (button), "toggled", G_CALLBACK
 		    (game_speed_cb), (gpointer) 4);
 
   button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -272,7 +282,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   if (properties->gamespeed == 3)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+  g_signal_connect (GTK_OBJECT (button), "toggled", G_CALLBACK
 		    (game_speed_cb), (gpointer) 3);
 
   button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -282,7 +292,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   if (properties->gamespeed == 2)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+  g_signal_connect (GTK_OBJECT (button), "toggled", G_CALLBACK
 		    (game_speed_cb), (gpointer) 2);
 
   button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -292,7 +302,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   if (properties->gamespeed == 1)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+  g_signal_connect (GTK_OBJECT (button), "toggled", G_CALLBACK
 		    (game_speed_cb), (gpointer) 1);
 
 
@@ -311,7 +321,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
     gtk_widget_set_sensitive (button, FALSE);
   if (properties->random)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+  g_signal_connect (GTK_OBJECT (button), "toggled", G_CALLBACK
 		    (random_order_cb), NULL);
 
   button = gtk_check_button_new_with_mnemonic (_("_Enable fake bonuses"));
@@ -321,14 +331,14 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
     gtk_widget_set_sensitive (button, FALSE);
   if (properties->fakes)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+  g_signal_connect (GTK_OBJECT (button), "toggled", G_CALLBACK
 		    (fake_bonus_cb), NULL);
 
   button = gtk_check_button_new_with_mnemonic (_("E_nable sounds"));
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   if (properties->sound)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+  g_signal_connect (GTK_OBJECT (button), "toggled", G_CALLBACK
 		    (sound_cb), NULL);
 
   table2 = gtk_table_new (3, 2, FALSE);
@@ -364,7 +374,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
     gtk_widget_set_sensitive (GTK_WIDGET (levelspinner), FALSE);
   gtk_table_attach_defaults (GTK_TABLE (table2), levelspinner, 1, 2, 0, 1);
   g_signal_connect (GTK_OBJECT (adjustment), "value_changed",
-		    GTK_SIGNAL_FUNC (start_level_cb), levelspinner);
+		    G_CALLBACK (start_level_cb), levelspinner);
 
   label2 = gtk_label_new_with_mnemonic (_("Number of _human players:"));
   gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
@@ -385,7 +395,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
   if (running || ggz_network_mode)
     gtk_widget_set_sensitive (num_human, FALSE);
   g_signal_connect (GTK_OBJECT (adjustment), "value_changed",
-		    GTK_SIGNAL_FUNC (num_worms_cb), num_human);
+		    G_CALLBACK (num_worms_cb), num_human);
 
   label2 = gtk_label_new_with_mnemonic (_("Number of _AI players:"));
   gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
@@ -406,13 +416,13 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
   if (running || ggz_network_mode)
     gtk_widget_set_sensitive (num_ai, FALSE);
   g_signal_connect (GTK_OBJECT (adjustment), "value_changed",
-		    GTK_SIGNAL_FUNC (num_worms_cb), num_ai);
+		    G_CALLBACK (num_worms_cb), num_ai);
 
   for (i = 0; i < NUMWORMS; i++) {
-    gchar *up_key;
-    gchar *down_key;
-    gchar *left_key;
-    gchar *right_key;
+    char up_key[64];
+    char down_key[64];
+    char left_key[64];
+    char right_key[64];
 
     buffer = g_strdup_printf ("%s %d", _("Worm"), i + 1);
     label = gtk_label_new (buffer);
@@ -425,23 +435,21 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
 
     frame = games_frame_new (_("Keyboard Controls"));
 
-    controls = games_controls_list_new ();
+    controls = games_controls_list_new (KEY_PREFERENCES_GROUP);
 
-    left_key = g_strdup_printf (KEY_WORM_LEFT, i);
-    right_key = g_strdup_printf (KEY_WORM_RIGHT, i);
-    up_key = g_strdup_printf (KEY_WORM_UP, i);
-    down_key = g_strdup_printf (KEY_WORM_DOWN, i);
+    g_snprintf (left_key, sizeof (left_key), KEY_WORM_LEFT, i);
+    g_snprintf (right_key, sizeof (right_key), KEY_WORM_RIGHT, i);
+    g_snprintf (up_key, sizeof (up_key), KEY_WORM_UP, i);
+    g_snprintf (down_key, sizeof (down_key), KEY_WORM_DOWN, i);
 
     games_controls_list_add_controls (GAMES_CONTROLS_LIST (controls),
-				      left_key, right_key, up_key, down_key,
+				      left_key, _("Move left"), GDK_Left,
+                                      right_key, _("Move right"), GDK_Right,
+                                      up_key, _("Move up"), GDK_Up,
+                                      down_key, _("Move down"), GDK_Down,
 				      NULL);
     gtk_container_add (GTK_CONTAINER (frame), controls);
 
-    g_free (left_key);
-    g_free (right_key);
-    g_free (up_key);
-    g_free (down_key);
-
     gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
 
     frame = games_frame_new (_("Options"));
@@ -471,7 +479,7 @@ gnibbles_preferences_cb (GtkWidget * widget, gpointer data)
     gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Purple"));
     gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), _("Gray"));
     g_signal_connect (GTK_OBJECT (omenu), "changed",
-		      GTK_SIGNAL_FUNC (set_worm_color_cb),
+		      G_CALLBACK (set_worm_color_cb),
 		      GINT_TO_POINTER (i));
     gtk_combo_box_set_active (GTK_COMBO_BOX (omenu),
 			      properties->wormprops[i]->color - WORMRED);
