@@ -18,7 +18,8 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
+#include <gtk/gtk.h>
 
 #include "gnibbles.h"
 #include "warp.h"
