@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import gtk, difflib,re
 DEBUG=False
 def debug (*args,**kwargs):
@@ -349,6 +350,11 @@ class UndoHistoryList (list):
         list.remove(self,obj)
         self.gui_update()
 
+    def clear (self):
+        while self: list.remove(self,self[0])
+        self.gui_update()
+        
+
 class MultipleUndoLists:
     """For tabbed interfaces and other places where it makes sense for one
     Action (menu item, etc.) to control multiple UndoLists (since presumably
