@@ -26,11 +26,11 @@
 
 #include <cairo.h>
 #include <glib.h>
-
-#include "block-cache.h"
-#include "blockops.h"
 #include <clutter-cairo/clutter-cairo.h>
 
+#include "blocks.h"
+#include "tetris.h"
+
 struct ThemeTableEntry {
 	const gchar *name;
 	const gchar *id;
@@ -40,54 +40,28 @@ extern const ThemeTableEntry ThemeTable[];
 
 class Renderer {
 public:
-	Renderer (ClutterActor * dst, Block ** src,
-		  int w, int h, int pxw, int pxh);
+	Renderer (gint pxw, gint pxh, bool initFromSubclass);
 	virtual ~ Renderer ();
-	virtual void render ();
 
-	void setTarget (ClutterActor *target);
+	void rescaleCache (gint pxw, gint pxh);
+	ClutterActor* getCacheCellById (gint id);
 
-	Block **data;
-	int width;
-	int height;
-	int pxwidth;
-	int pxheight;
+	gint pxwidth;
+	gint pxheight;
 protected:
-	GnometrisBlockCache *block_cache;
-	ClutterActor *target;
-
-	virtual void drawCell (cairo_t * cr, gint x, gint y);
-	virtual void drawForeground (cairo_t * cr);
+	ClutterActor* cache[NCOLOURS];
+	virtual void drawCell (cairo_t * cr, guint color);
 };
 
-Renderer *rendererFactory (gint id, ClutterActor * dst,
-			   Block ** src, int w,
-			   int h, int pxw, int pxh);
+Renderer *rendererFactory (gint id, gint pxw, gint pxh);
 gint themeNameToNumber (const gchar * id);
 
-class JoinedUp:public Renderer {
-public:
-	JoinedUp (ClutterActor * dst, Block ** src,
-	int w, int h, int pxw, int pxh):Renderer (dst, src, w, h, pxw,
-						  pxh) {}
-protected:
-	virtual void drawCell (cairo_t * cr, gint x, gint y);
-
-private:
-	double border;
-	void drawInnerCorner (cairo_t * cr);
-	void drawOuterCorner (cairo_t * cr);
-	void drawHEdge (cairo_t * cr);
-	void drawVEdge (cairo_t * cr);
-};
-
 class TangoBlock:public Renderer {
 public:
-	TangoBlock (ClutterActor * dst, Block ** src,
-		    int w, int h, int pxw, int pxh, gboolean grad);
+	TangoBlock (gint pxw, gint pxh, gboolean grad);
 
 protected:
-	virtual void drawCell (cairo_t * cr, gint x, gint y);
+	virtual void drawCell (cairo_t * cr, guint color);
 	gboolean usegrads;
 
 private:
