@@ -30,10 +30,11 @@
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include <config.h>
-#include <gnome.h>
-#include <gconf/gconf-client.h>
 
-#include <games-frame.h>
+#include <gtk/gtk.h>
+
+#include <libgames-support/games-frame.h>
+#include <libgames-support/games-conf.h>
 
 #include "yahtzee.h"
 #include "gyahtzee.h"
@@ -48,7 +49,9 @@ static int OriginalNumberOfComputers = -1;
 static int OriginalNumberOfHumans = -1;
 static GameType NewGameType  = GAME_YAHTZEE;
 
-static int tmpDoDelay, tmpDisplayComputerThoughts;
+static int tmpDoDelay = -1, tmpDisplayComputerThoughts;
+static int skill_level;
+extern int NUM_TRIALS;
 
 extern GtkWidget *window;
 typedef struct game_type_table_t {
@@ -63,6 +66,19 @@ static GameTypeTable game_type_table[] = {
 
 #define GAME_TYPE_TABLE_SIZE (sizeof(game_type_table) / sizeof(GameTypeTable))
 
+typedef struct skill_level_table_t {
+    int level;
+    int trials;
+} SkillLevelTable;
+
+static SkillLevelTable skill_level_table[] = {
+    { 0, 10 },
+    { 1, 100 },
+    { 2, 1000 }
+};
+
+#define SKILL_LEVEL_TABLE_SIZE (sizeof(skill_level_table) / sizeof(SkillLevelTable))
+
 static const gchar *game_type_name(GameType type)
 {
     gint ii = 0;
@@ -115,18 +131,21 @@ WarnNumPlayersChanged (void)
 static void
 do_setup (GtkWidget * widget, gpointer data)
 {
-  GConfClient *client;
-  GError *err = NULL;
-  GSList *name_list = NULL;
   const gchar  *type_name = NULL;
   int i;
+  char **player_names;
+  gsize n_player_names;
 
   NumberOfComputers =
     gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (ComputerSpinner));
   NumberOfHumans =
     gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (HumanSpinner));
+    
+  if (tmpDoDelay == -1)
+    tmpDoDelay = DoDelay;
+  else
+    DoDelay = tmpDoDelay;
 
-  DoDelay = tmpDoDelay;
   DisplayComputerThoughts = tmpDisplayComputerThoughts;
 
   for (i = 0; i < MAX_NUMBER_OF_PLAYERS; i++) {
@@ -137,56 +156,33 @@ do_setup (GtkWidget * widget, gpointer data)
 
     if (i < NumberOfPlayers)
       score_list_set_column_title (ScoreList, i + 1, players[i].name);
-    name_list = g_slist_append (name_list, players[i].name);
   }
 
   setupdialog_destroy (setupdialog, 1);
 
-  client = gconf_client_get_default ();
-  gconf_client_set_list (client, "/apps/gtali/PlayerNames",
-			 GCONF_VALUE_STRING, name_list, &err);
-  g_slist_free (name_list);
-  if (err) {
-    g_warning (G_STRLOC ": gconf error: %s\n", err->message);
-    g_error_free (err);
-    err = NULL;
+  n_player_names = MAX_NUMBER_OF_PLAYERS;
+  player_names = g_newa (char *, n_player_names);
+  for (i = 0; i < MAX_NUMBER_OF_PLAYERS; ++i) {
+    player_names[i] = players[i].name;
   }
 
-  gconf_client_set_int (client, "/apps/gtali/NumberOfComputerOpponents",
-			NumberOfComputers, &err);
-  if (err) {
-    g_warning (G_STRLOC ": gconf error: %s\n", err->message);
-    g_error_free (err);
-    err = NULL;
-  }
+  games_conf_set_string_list (NULL, KEY_PLAYER_NAMES,
+                              (const char * const *) player_names,
+                              n_player_names);
 
-  gconf_client_set_int (client, "/apps/gtali/NumberOfHumanOpponents",
-			NumberOfHumans, &err);
-  if (err) {
-    g_warning (G_STRLOC ": gconf error: %s\n", err->message);
-    g_error_free (err);
-    err = NULL;
-  }
+  games_conf_set_integer (NULL, KEY_NUMBER_OF_COMPUTERS, NumberOfComputers);
 
-  gconf_client_set_bool (client, "/apps/gtali/DelayBetweenRolls",
-			 DoDelay, &err);
-  if (err) {
-    g_warning (G_STRLOC ": gconf error: %s\n", err->message);
-    g_error_free (err);
-    err = NULL;
-  }
+  games_conf_set_integer (NULL, KEY_NUMBER_OF_HUMANS, NumberOfHumans);
+
+  games_conf_set_boolean (NULL, KEY_DELAY_BETWEEN_ROLLS, DoDelay);
 
   type_name = game_type_name(NewGameType);
   if (type_name) {
-    gconf_client_set_string (client, "/apps/gtali/GameType", type_name,
-                             &err);
-    if (err) {
-      g_warning (G_STRLOC ": gconf error: %s\n", err->message);
-      g_error_free (err);
-      err = NULL;
-    }
+    games_conf_set_string (NULL, KEY_GAME_TYPE, type_name);
   }
 
+  games_conf_set_integer (NULL, KEY_NUMTRIALS, NUM_TRIALS);
+
   if (((NumberOfComputers != OriginalNumberOfComputers)
        || (NumberOfHumans != OriginalNumberOfHumans) 
        || (NewGameType != game_type))
@@ -243,6 +239,16 @@ SetGameType (GtkWidget *widget, gpointer *data)
     return FALSE;
 }
 
+static gint
+SetSkillLevel (GtkWidget *widget, gpointer *data)
+{
+    skill_level = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+    if (skill_level >= 0 && skill_level < SKILL_LEVEL_TABLE_SIZE) {
+        NUM_TRIALS = skill_level_table[skill_level].trials;
+    }
+    return FALSE;
+}
+
 gint
 setup_game (GtkAction * action, gpointer data)
 {
@@ -294,7 +300,7 @@ setup_game (GtkAction * action, gpointer data)
 
   gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
   HumanAdj = gtk_adjustment_new ((gfloat) NumberOfHumans, 1.0,
-				 6.0, 1.0, 6.0, 1.0);
+				 6.0, 1.0, 6.0, 0.0);
   HumanSpinner = gtk_spin_button_new (GTK_ADJUSTMENT (HumanAdj), 10, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), HumanSpinner);
 
@@ -326,7 +332,7 @@ setup_game (GtkAction * action, gpointer data)
   gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
 
   ComputerAdj = gtk_adjustment_new ((gfloat) NumberOfComputers,
-				    0.0, 5.0, 1.0, 5.0, 1.0);
+				    0.0, 5.0, 1.0, 5.0, 0.0);
   ComputerSpinner = gtk_spin_button_new (GTK_ADJUSTMENT (ComputerAdj), 10, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), ComputerSpinner);
 
@@ -334,14 +340,30 @@ setup_game (GtkAction * action, gpointer data)
 		    G_CALLBACK (MaxPlayersCheck), ComputerAdj);
   gtk_box_pack_start (GTK_BOX (box2), ComputerSpinner, TRUE, TRUE, 0);
 
+  box2 = gtk_hbox_new(FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(box), box2, FALSE, FALSE, 0);
+  label = gtk_label_new_with_mnemonic (_("_Difficulty:"));
+  gtk_box_pack_start(GTK_BOX(box2), label, FALSE, FALSE, 0);
+  combo = gtk_combo_box_new_text();
+  gtk_combo_box_append_text(GTK_COMBO_BOX(combo), C_("difficulty", "Easy"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(combo), C_("difficulty", "Medium"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(combo), C_("difficulty", "Hard"));
+  skill_level = 0;
+  while (NUM_TRIALS > skill_level_table[skill_level].trials &&
+         skill_level < (SKILL_LEVEL_TABLE_SIZE - 1)) skill_level++;
+  gtk_combo_box_set_active(GTK_COMBO_BOX(combo), skill_level);
+  g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (SetSkillLevel),
+                    combo);
+  gtk_box_pack_start(GTK_BOX(box2), combo, FALSE, FALSE, 0);
+
     /*--- Combo (yahtzee or kismet style ----*/
 
   frame = games_frame_new (_("Game Type"));
   gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 2, 3, GTK_FILL,
                     GTK_FILL | GTK_EXPAND, 0, 0);
   combo = gtk_combo_box_new_text();
-  gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Regular"));
-  gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("Colors"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(combo), C_("game type", "Regular"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(combo), C_("game type", "Colors"));
   gtk_combo_box_set_active(GTK_COMBO_BOX(combo), game_type);
   NewGameType = game_type;
   g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (SetGameType),
