@@ -42,8 +42,8 @@
 #endif /* WITH_SMCLIENT */
 
 #ifdef GGZ_CLIENT
-#include <games-dlg-chat.h>
-#include <games-dlg-players.h>
+#include <libgames-support/games-dlg-chat.h>
+#include <libgames-support/games-dlg-players.h>
 #include "ggz-network.h"
 #include <ggz-embed.h>
 #endif /* WITH GGZ_CLIENT */
@@ -73,7 +73,8 @@ GdkPixmap *buffer_pixmap = NULL;
 GdkPixmap *tiles_pixmap = NULL;
 GdkPixmap *tiles_mask = NULL;
 
-gint flip_pixmaps_id = 0;
+static gint flip_pixmaps_id = 0;
+static gint flip_animation_speed = PIXMAP_FLIP_DELAY;
 guint statusbar_id;
 guint black_computer_level;
 guint white_computer_level;
@@ -243,7 +244,7 @@ undo_move_cb (GtkWidget * widget, gpointer data)
   }
 
   gui_status ();
-  tiles_to_flip = 1;
+  start_animation ();
   check_computer_players ();
 }
 
@@ -428,18 +429,44 @@ load_pixmaps (void)
 						  &tiles_pixmap,
 						  &tiles_mask, 127);
 
-  gdk_pixbuf_unref (image);
+  g_object_unref (image);
   g_free (fname);
 }
 
+void
+set_animation_speed (gint speed)
+{
+    flip_animation_speed = speed;
+    if (flip_pixmaps_id) {
+        g_source_remove (flip_pixmaps_id);
+        flip_pixmaps_id = g_timeout_add (flip_animation_speed, flip_pixmaps, NULL);
+    }
+}
+
+void
+start_animation (void)
+{
+    if (flip_pixmaps_id == 0)
+        flip_pixmaps_id = g_timeout_add (flip_animation_speed, flip_pixmaps, NULL);
+    tiles_to_flip = 1;
+}
+
+void
+stop_animation (void)
+{
+    tiles_to_flip = 0;
+}
+
 gint
 flip_pixmaps (gpointer data)
 {
   guint i, j;
   guint flipped_tiles = 0;
 
-  if (!tiles_to_flip)
-    return TRUE;
+  if (!tiles_to_flip) {
+      flip_pixmaps_id = 0;
+      return FALSE;
+  }
 
   for (i = 0; i < 8; i++)
     for (j = 0; j < 8; j++) {
@@ -505,7 +532,7 @@ flip_pixmaps (gpointer data)
     }
 
   if (!flipped_tiles)
-    tiles_to_flip = 0;
+    stop_animation ();
 
   return TRUE;
 }
@@ -952,20 +979,9 @@ main (int argc, char **argv)
   EggSMClient *sm_client;
 #endif /* WITH_SMCLIENT */
 
-#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
-  /* If we're going to use gnome-vfs, we need to init threads before
-   * calling any glib functions.
-   */
-  g_thread_init (NULL);
-#endif
-
   if (!games_runtime_init ("iagno"))
     return 1;
 
-  bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
-
   context = g_option_context_new (NULL);
 #if GLIB_CHECK_VERSION (2, 12, 0)
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
