@@ -48,7 +48,7 @@ typedef union {
     guint8 face_down : 1;
     guint8 suit : 2;
     guint8 rank : 4;
-  } GNOME_GAMES_GNUC_PACKED attr;
+  } attr;
 } GNOME_GAMES_GNUC_PACKED Card;
 
 typedef int _games_card_size_assert[sizeof (Card) == sizeof (guint8) ? 1 : -1]; /* static assertion */
@@ -65,15 +65,6 @@ typedef int _games_card_size_assert[sizeof (Card) == sizeof (guint8) ? 1 : -1];
 
 /* Some defines */
 
-#define SCALABLE_CARDS_DIR    COMMON_DATADIR G_DIR_SEPARATOR_S "cards"
-#define PRERENDERED_CARDS_DIR COMMON_DATADIR G_DIR_SEPARATOR_S "card-themes"
-
-#ifdef HAVE_HILDON
-#define GAMES_CARD_THEME_DEFAULT "paris"
-#else
-#define GAMES_CARD_THEME_DEFAULT "bonded"
-#endif
-
 typedef enum {
   /* Cards */
   GAMES_CARD_JOKER = 0,
@@ -113,12 +104,18 @@ int games_card_get_node_by_suit_and_rank_snprintf (char *buffer,
                                                    int suit,
                                                    int rank);
 
+int games_card_get_node_by_id_snprintf (char *buffer,
+                                        gsize bufsize,
+                                        int card_id);
+
 int games_card_get_name_by_id_snprintf (char *buffer,
                                         gsize bufsize,
                                         int card_id);
 
 char * games_card_get_name_by_id (int card_id);
 
+const char * games_card_get_localised_rank_symbol (int rank);
+
 G_END_DECLS
 
 #endif /* !GAMES_CARD_H */
