@@ -1,6 +1,4 @@
 /*
- * games-scores.c
- *
  * Copyright 2005 Callum McKenzie
  *
  * This library is free software; you can redistribute it and'or modify
@@ -31,18 +29,19 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+#include <glib/gi18n.h>
+
 #include "games-scores-backend.h"
 #include "games-score.h"
 #include "games-scores.h"
 
 /* The local version of the GamesScoresCategory. */
 typedef struct {
-  gchar *key;
-  gchar *name;
+  GamesScoresCategory category;
   GamesScoresBackend *backend;
-} GamesScoresCategoryPrivate;
+} GamesScoresCategoryInternal;
 
-typedef struct _GamesScoresPrivate {
+struct _GamesScoresPrivate {
   GHashTable *categories;
   GSList *catsordered;
   gchar *currentcat;
@@ -52,14 +51,14 @@ typedef struct _GamesScoresPrivate {
   gint last_score_position;
   GamesScoreValue last_score_value;
   GamesScoreStyle style;
-  GamesScoresCategoryPrivate dummycat;
-} GamesScoresPrivate;
+  GamesScoresCategoryInternal dummycat;
+};
 
 static void
-games_scores_category_free (GamesScoresCategoryPrivate * cat)
+games_scores_category_free (GamesScoresCategoryInternal *cat)
 {
-  g_free (cat->key);
-  g_free (cat->name);
+  g_free (cat->category.key);
+  g_free (cat->category.name);
   if (cat->backend)
     g_object_unref (cat->backend);
   g_free (cat);
@@ -72,23 +71,24 @@ games_scores_category_free (GamesScoresCategoryPrivate * cat)
  * Retrieves the current category and make sure it is in a state to be used.
  *
  **/
-static GamesScoresCategoryPrivate *
+static GamesScoresCategoryInternal *
 games_scores_get_current (GamesScores * self)
 {
-  GamesScoresCategoryPrivate *cat;
+  GamesScoresPrivate *priv = self->priv;
+  GamesScoresCategoryInternal *cat;
 
-  if (self->priv->currentcat == NULL) {
+  if (priv->currentcat == NULL) {
     /* We have a single, anonymous, category. */
-    cat = &(self->priv->dummycat);
+    cat = &(priv->dummycat);
   } else {
-    cat =
-      g_hash_table_lookup (self->priv->categories, self->priv->currentcat);
+    cat = g_hash_table_lookup (priv->categories, priv->currentcat);
+    if (!cat)
+      return NULL;
   }
 
   if (cat->backend == NULL) {
-    cat->backend =
-      games_scores_backend_new (self->priv->style, self->priv->basename,
-				cat->key);
+    cat->backend = games_scores_backend_new (priv->style, priv->basename,
+                                             cat->category.key);
   }
 
   return cat;
@@ -101,84 +101,109 @@ games_scores_get_current (GamesScores * self)
 G_DEFINE_TYPE (GamesScores, games_scores, G_TYPE_OBJECT);
 
 /** 
- * new:
- * @description: A GamesScoresDescription structure with the information
- *               about this games scoring system. 
+ * games_scores_new:
+ * @app_name: the (old) app name (for backward compatibility),
+ *   used as the basename of the category filenames
+ * @categories: the score categories, or %NULL to use an anonymous category
+ * @n_categories: the number of category entries in @categories
+ * @categories_context: the translation context to use for the category names,
+ *   or %NULL to use no translation context
+ * @categories_domain: the translation domain to use for the category names,
+ *   or %NULL to use the default domain
+ * @default_category: the key of the default category, or %NULL
+ * @style: the category style
+ * 
  *
- * Create an object to handle a set of scores. Normally you will make one
- * global object. Creating and destroying these objects is inefficient. 
- * Using multipl objects referring to the same set of scores at the same
- * time should work but is unnecessary liable to be buggy.
+ * Returns: a new #GamesScores object
  */
 GamesScores *
-games_scores_new (const GamesScoresDescription * description)
+games_scores_new (const char *app_name,
+                  const GamesScoresCategory *categories,
+                  int n_categories,
+                  const char *categories_context,
+                  const char *categories_domain,
+                  int default_category_index,
+                  GamesScoreStyle style)
 {
   GamesScores *self;
-  const GamesScoresCategory *cats;
+  GamesScoresPrivate *priv;
 
   self = GAMES_SCORES (g_object_new (GAMES_TYPE_SCORES, NULL));
+  priv = self->priv;
 
   /* FIXME: Input sanity checks. */
 
-  self->priv->categories =
-    g_hash_table_new_full (g_str_hash, g_str_equal,
-			   g_free,
-			   (GDestroyNotify) games_scores_category_free);
+  priv->categories = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                            g_free,
+                                            (GDestroyNotify) games_scores_category_free);
 
   /* catsordered is a record of the ordering of the categories. 
    * Its data is shared with the hash table. */
-  self->priv->catsordered = NULL;
+  priv->catsordered = NULL;
 
-  if (description->categories) {
-    cats = description->categories;
-    while (cats->key) {
-      games_scores_add_category (self, cats->key, cats->name);
-      cats++;
+  if (n_categories > 0) {
+    int i;
+
+    g_return_val_if_fail (default_category_index >= 0 && default_category_index < n_categories, NULL);
+
+    for (i = 0; i < n_categories; ++i) {
+      const GamesScoresCategory *category = &categories[i];
+      const char *display_name;
+
+      if (categories_context) {
+        display_name = g_dpgettext2 (categories_domain, categories_context, category->name);
+      } else {
+        display_name = dgettext (categories_domain, category->name);
+      }
+
+      games_scores_add_category (self, category->key, display_name);
     }
 
-    self->priv->defcat = g_strdup (description->deflt);
-    self->priv->currentcat = g_strdup (self->priv->defcat);
+    priv->defcat = g_strdup (categories[default_category_index].key);
+    priv->currentcat = g_strdup (priv->defcat);
   } else {
-    self->priv->currentcat = NULL;
-    self->priv->defcat = NULL;
-    self->priv->catsordered = NULL;
+    priv->currentcat = NULL;
+    priv->defcat = NULL;
   }
 
-  self->priv->basename = g_strdup (description->basename);
+  priv->basename = g_strdup (app_name);
   /* FIXME: Do some sanity checks on the default and the like. */
 
-  self->priv->style = description->style;
+  priv->style = style;
 
   /* Set up the anonymous category for use when no categories are specified. */
-  self->priv->dummycat.key = "";
-  self->priv->dummycat.name = "";
-  self->priv->dummycat.backend = NULL;
+  priv->dummycat.category.key = "";
+  priv->dummycat.category.name = "";
+  priv->dummycat.backend = NULL;
 
   return self;
 }
 
 /**
- * add_category:
- * @scores: A scores object.
- * @key: The key for the new category.
- * @name: The player-visible label for the new category.
+ * games_scores_add_category:
+ * @self:
+ * @key: the key for the new category
+ * @name: the user visible label for the new category
  *
  * Add a new category after initialisation. key and name are copied into
  * internal structures. The scores dialog is not currently updated.
  *
  **/
 void
-games_scores_add_category (GamesScores * self, gchar * key, gchar * name)
+games_scores_add_category (GamesScores *self,
+                           const char *key,
+                           const char *name)
 {
-  GamesScoresCategoryPrivate *cat;
+  GamesScoresPrivate *priv = self->priv;
+  GamesScoresCategoryInternal *cat;
 
-  cat = g_new (GamesScoresCategoryPrivate, 1);
-  cat->key = g_strdup (key);
-  cat->name = g_strdup (name);
+  cat = g_new (GamesScoresCategoryInternal, 1);
+  cat->category.key = g_strdup (key);
+  cat->category.name = g_strdup (name);
   cat->backend = NULL;
 
-  g_hash_table_insert (self->priv->categories, g_strdup (key), cat);
-  self->priv->catsordered = g_slist_append (self->priv->catsordered, cat);
+  g_hash_table_insert (priv->categories, g_strdup (key), cat);
+  priv->catsordered = g_slist_append (priv->catsordered, cat);
 }
 
 /**
@@ -196,16 +221,15 @@ games_scores_add_category (GamesScores * self, gchar * key, gchar * name)
 void
 games_scores_set_category (GamesScores * self, gchar * category)
 {
+  GamesScoresPrivate *priv = self->priv;
+
   g_return_if_fail (self != NULL);
-  g_return_if_fail (self->priv != NULL);
 
   if (category == NULL)
-    category = self->priv->defcat;
+    category = priv->defcat;
 
-  if (self->priv->currentcat)
-    g_free (self->priv->currentcat);
-
-  self->priv->currentcat = g_strdup (category);
+  g_free (priv->currentcat);
+  priv->currentcat = g_strdup (category);
 
   /* FIXME: Check validity of category (Null, the same as current, 
    * is actually a category) then just set it in the structure. */
@@ -226,13 +250,13 @@ games_scores_set_category (GamesScores * self, gchar * category)
 gint
 games_scores_add_score (GamesScores * self, GamesScoreValue score)
 {
+  GamesScoresPrivate *priv = self->priv;
   GamesScore *fullscore;
-  GamesScoresCategoryPrivate *cat;
+  GamesScoresCategoryInternal *cat;
   gint place, n;
   GList *s, *scores_list;
 
   g_return_val_if_fail (self != NULL, 0);
-  g_return_val_if_fail (self->priv != NULL, 0);
 
   fullscore = games_score_new ();
   fullscore->value = score;
@@ -251,7 +275,7 @@ games_scores_add_score (GamesScores * self, GamesScoreValue score)
     n++;
 
     /* If beat someone in the list, add us there. */
-    if (games_score_compare (self->priv->style, oldscore, fullscore) < 0) {
+    if (games_score_compare (priv->style, oldscore, fullscore) < 0) {
       scores_list = g_list_insert_before (scores_list, s,
 					  games_score_dup (fullscore));
       place = n;
@@ -281,15 +305,15 @@ games_scores_add_score (GamesScores * self, GamesScoreValue score)
   if (games_scores_backend_set_scores (cat->backend, scores_list) == FALSE)
     place = 0;
 
-  self->priv->last_score_significant = place > 0;
-  self->priv->last_score_position = place;
-  self->priv->last_score_value = score;
+  priv->last_score_significant = place > 0;
+  priv->last_score_position = place;
+  priv->last_score_value = score;
 
   return place;
 }
 
 /**
- * games_scores_update_score:
+ * games_scores_update_score_name:
  * @self: A scores object.
  * @new_name: The new name to use.
  *
@@ -300,25 +324,27 @@ games_scores_add_score (GamesScores * self, GamesScoreValue score)
  *
  **/
 void
-games_scores_update_score (GamesScores * self, gchar * new_name)
+games_scores_update_score_name (GamesScores * self, gchar * new_name, gchar * old_name)
 {
-  GamesScoresCategoryPrivate *cat;
+  GamesScoresPrivate *priv = self->priv;
+  GamesScoresCategoryInternal *cat;
   GList *s, *scores_list;
   gint n, place;
-  gchar *old_name;
   GamesScore *sc;
   GamesScoreValue score;
 
   g_return_if_fail (self != NULL);
-  g_return_if_fail (self->priv != NULL);
 
-  place = self->priv->last_score_position;
-  score = self->priv->last_score_value;
+  place = priv->last_score_position;
+  score = priv->last_score_value;
 
   if (place == 0)
     return;
 
-  old_name = g_strdup (g_get_real_name ());
+  if (old_name)
+      old_name = g_strdup (old_name); /* Make copy so we can free it later */
+  else
+      old_name = g_strdup (g_get_real_name ());
 
   cat = games_scores_get_current (self);
 
@@ -335,7 +361,7 @@ games_scores_update_score (GamesScores * self, gchar * new_name)
 
   while ((n >= place) && (s != NULL)) {
     sc = (GamesScore *) (s->data);
-    if ((games_score_compare_values (self->priv->style, sc->value, score) ==
+    if ((games_score_compare_values (priv->style, sc->value, score) ==
 	 0) && (g_utf8_collate (old_name, sc->name) == 0)) {
       g_free (sc->name);
       sc->name = g_strdup (new_name);
@@ -350,6 +376,23 @@ games_scores_update_score (GamesScores * self, gchar * new_name)
   g_free (old_name);
 }
 
+/**
+ * games_scores_update_score:
+ * @self: A scores object.
+ * @new_name: The new name to use.
+ *
+ * By default add_score uses the current user name. This routine updates
+ * that name. There are a few wrinkles: the score may have moved since we
+ * got the original score. Use in normal code is discouraged, it is here 
+ * to be used by GamesScoresDialog.
+ *
+ **/
+void
+games_scores_update_score (GamesScores * self, gchar * new_name)
+{
+    games_scores_update_score_name (self, new_name, NULL);
+}
+
 /**
  * get:
  * @self: A scores object.
@@ -362,11 +405,10 @@ games_scores_update_score (GamesScores * self, gchar * new_name)
 GList *
 games_scores_get (GamesScores * self)
 {
-  GamesScoresCategoryPrivate *cat;
+  GamesScoresCategoryInternal *cat;
   GList *scores;
 
   g_return_val_if_fail (self != NULL, NULL);
-  g_return_val_if_fail (self->priv != NULL, NULL);
 
   cat = games_scores_get_current (self);
 
@@ -379,7 +421,7 @@ games_scores_get (GamesScores * self)
 }
 
 /**
- * category_foreach:
+ * _games_scores_category_foreach:
  * @self: A scores object.
  * @func: A function to call.
  * @userdata: Arbitrary data.
@@ -390,23 +432,17 @@ games_scores_get (GamesScores * self)
  *
  **/
 void
-games_scores_category_foreach (GamesScores * self,
-			       GamesScoresCategoryForeachFunc func,
-			       gpointer userdata)
+_games_scores_category_foreach (GamesScores * self,
+                                GamesScoresCategoryForeachFunc func,
+                                gpointer userdata)
 {
-  GSList *list;
-  GamesScoresCategory temp;
+  GamesScoresPrivate *priv = self->priv;
+  GSList *l;
 
   g_return_if_fail (self != NULL);
-  g_return_if_fail (self->priv != NULL);
 
-  list = self->priv->catsordered;
-  while (list) {
-    temp.key = ((GamesScoresCategoryPrivate *) list->data)->key;
-    temp.name = ((GamesScoresCategoryPrivate *) list->data)->name;
-
-    (*func) (&temp, userdata);
-    list = g_slist_next (list);
+  for (l = priv->catsordered; l != NULL; l = l->next) {
+    func ((GamesScoresCategory*) l->data, userdata);
   }
 }
 
@@ -420,10 +456,11 @@ games_scores_category_foreach (GamesScores * self,
 GamesScoreStyle
 games_scores_get_style (GamesScores * self)
 {
-  g_return_val_if_fail (self != NULL, 0);
-  g_return_val_if_fail (self->priv != NULL, 0);
+  GamesScoresPrivate *priv = self->priv;
 
-  return self->priv->style;
+  g_return_val_if_fail (self != NULL, 0);
+
+  return priv->style;
 }
 
 /**
@@ -438,23 +475,25 @@ games_scores_get_style (GamesScores * self)
 const gchar *
 games_scores_get_category (GamesScores * self)
 {
-  g_return_val_if_fail (self != NULL, NULL);
-  g_return_val_if_fail (self->priv != NULL, NULL);
+  GamesScoresPrivate *priv = self->priv;
 
-  return self->priv->currentcat;
+  g_return_val_if_fail (self != NULL, NULL);
+
+  return priv->currentcat;
 }
 
 static void
 games_scores_init (GamesScores * self)
 {
+  GamesScoresPrivate *priv;
+
   /* Most of the work is done in the _new method. */
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GAMES_TYPE_SCORES,
-					    GamesScoresPrivate);
+  priv = self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GAMES_TYPE_SCORES, GamesScoresPrivate);
 
-  self->priv->last_score_significant = FALSE;
-  self->priv->last_score_position = 0;
-  self->priv->last_score_value.plain = 0;
+  priv->last_score_significant = FALSE;
+  priv->last_score_position = 0;
+  priv->last_score_value.plain = 0;
 }
 
 static void
